import logging
import os
from logging.handlers import RotatingFileHandler
from typing import Literal



class Logger:
    """
    Logger class to log messages to a file and optionally to the console.
    """
    def __init__(
        self,
        logname: str,
        level: str = "INFO",
        disable_file: bool = False,
        disable_stdout: bool = False,
        log_file_path: str = None,
    ):
        """
        args:
        logname (str): The name of the logger.
        level (str): The logging level. Default is INFO.
        disable_file (bool): If True, the logger will not log to a file. Default is False.
        disable_stdout (bool): If True, the logger will not log to the console. Default is False.
        log_file_path (str): The path to the log file. If None, the log file will be saved as {logname}.log.
        """
        self.logname = logname
        self.level = getattr(logging, level.upper(), logging.INFO)
        self.file_log = log_file_path or f"{logname}.log"
        self.maxBytes = 1024 * 1024 * 10  # 10 MB
        self.format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"

        self.file_logger = self._init_file_logger()
        self.std_out_logger = self._init_stdout_logger()
        self.file_logger.disabled = disable_file
        self.std_out_logger.disabled = disable_stdout

    def __call__(
        self,
        msg: str,
        level: Literal["info", "debug", "warning"],
        std_out: bool = False,
    ):
        self.log(msg=msg, level=level, std_out=std_out)

    def _init_file_logger(self):
        logger = logging.getLogger(self.logname)
        logger.setLevel(self.level)

        if not any(
            isinstance(handler, RotatingFileHandler) for handler in logger.handlers
        ):
            logging_handler = RotatingFileHandler(
                filename=self.file_log,
                mode="a",
                maxBytes=self.maxBytes,
                backupCount=2,
                encoding=None,
                delay=False,
            )
            logging_handler.setFormatter(logging.Formatter(self.format))
            logger.addHandler(logging_handler)

        return logger

    def _init_stdout_logger(self):
        stdout_logger = logging.getLogger(f"{self.logname}_stdout")
        stdout_logger.setLevel(self.level)

        if not any(
            isinstance(handler, logging.StreamHandler)
            for handler in stdout_logger.handlers
        ):
            stdout_handler = logging.StreamHandler()
            stdout_handler.setFormatter(logging.Formatter(self.format))
            stdout_logger.addHandler(stdout_handler)

        return stdout_logger

    def log(
        self,
        msg: str,
        level: Literal["info", "debug", "warning"],
        std_out: bool = False,
    ):
        log_method = {
            "info": logging.INFO,
            "debug": logging.DEBUG,
            "warning": logging.WARNING,
        }.get(level, logging.INFO)

        self.file_logger.log(log_method, msg)
        if std_out:
            self.std_out_logger.log(log_method, msg)

    def info(self, msg: str, std_out: bool = False):
        self.log(msg, "info", std_out)
    
    def debug(self, msg: str, std_out: bool = False):
        self.log(msg, "debug", std_out)
        
    def warning(self, msg: str, std_out: bool = False):
        self.log(msg, "warning", std_out)
        
    def error(self, msg: str, std_out: bool = False):
        red_error_color = "\033[91m"
        endc = "\033[0m"
        self.log(f"Error: {msg}", "warning", std_out)